@@ -35,6 +35,9 @@ struct io_u {
 #endif
 #ifdef FIO_HAVE_BINJECT
 		struct b_user_cmd buc;
+#endif
+#ifdef FIO_HAVE_RDMA
+		struct ibv_mr *mr;
 #endif
 		void *mmap_data;
 	};
@@ -48,6 +51,11 @@ struct io_u {
 	unsigned long buflen;
 	unsigned long long offset;
 
+	/*
+	 * Initial seed for generating the buffer contents
+	 */
+	unsigned long rand_seed;
+
 	/*
 	 * IO engine state, may be different from above when we get
 	 * partial transfers / residual data counts
@@ -125,9 +133,10 @@ enum fio_ioengine_flags {
 	FIO_NODISKUTIL  = 1 << 4,       /* diskutil can't handle filename */
 	FIO_UNIDIR	= 1 << 5,	/* engine is uni-directional */
 	FIO_NOIO	= 1 << 6,	/* thread does only pseudo IO */
-	FIO_SIGQUIT	= 1 << 7,	/* needs SIGQUIT to exit */
+	FIO_SIGTERM	= 1 << 7,	/* needs SIGTERM to exit */
 	FIO_PIPEIO	= 1 << 8,	/* input/output no seekable */
 	FIO_BARRIER	= 1 << 9,	/* engine supports barriers */
+	FIO_MEMALIGN	= 1 << 10,	/* engine wants aligned memory */
 };
 
 /*
